#! /usr/bin/env node
var sh = require('shelljs');
var path = require('path');
var fs = require('fs');
var inquirer = require('inquirer');
var colorsTerminal = require('colors');
var os = require('os');


const name = process.argv.slice(-1)[0];

let installLibCommandLine = `npm install --save `

async function createFolder() {

  const listFolders = [
    { label: 'src', path: `${sh.pwd().stdout}/src`, options: { recursive: true }, isCreate: true },

    { label: 'assets', path: `${sh.pwd().stdout}/src/assets`, options: { recursive: true }, isCreate: true },
    { label: 'assets', path: `${sh.pwd().stdout}/src/assets/data`, options: { recursive: true }, isCreate: true },

    { label: 'configs', path: `${sh.pwd().stdout}/src/configs`, options: { recursive: true }, isCreate: true },

    { label: 'routers', path: `${sh.pwd().stdout}/src/routers`, options: { recursive: true }, isCreate: true },

    { label: 'db', path: `${sh.pwd().stdout}/src/db`, options: { recursive: true }, isCreate: true },
    { label: 'db', path: `${sh.pwd().stdout}/src/db/models`, options: { recursive: true }, isCreate: true },
    { label: 'db', path: `${sh.pwd().stdout}/src/db/migrations`, options: { recursive: true }, isCreate: true },
    { label: 'db', path: `${sh.pwd().stdout}/src/db/seeds`, options: { recursive: true }, isCreate: true },

    { label: 'core', path: `${sh.pwd().stdout}/src/core`, options: { recursive: true }, isCreate: true },

    { label: 'utils', path: `${sh.pwd().stdout}/src/utils`, options: { recursive: true }, isCreate: true },

    { label: 'debugging', path: `${sh.pwd().stdout}/debugging`, options: { recursive: true }, isCreate: true },
  ];

  listFolders.forEach(element => {
    if (element.isCreate) {
      console.log(colorsTerminal.green('✔'), '[Project Structure]', element.label);
      try {
        fs.mkdirSync(element.path, element.options);
      } catch (err) {
        console.warn(err)
      }
    }
  });
}

async function createFiles() {
  const listFiles = [
    { label: 'src', path: `${sh.pwd().stdout}/src/app.js`, options: { content: '' } },

    { label: 'assets', path: `${sh.pwd().stdout}/src/assets/index.js`, options: { content: '' } },
    { label: 'assets', path: `${sh.pwd().stdout}/src/assets/package.json`, options: { content: JSON.stringify({ name: '@assets' }) } },
    { label: 'assets', path: `${sh.pwd().stdout}/src/assets/data/index.js`, options: { content: '' } },

    { label: 'configs', path: `${sh.pwd().stdout}/src/configs/index.js`, options: { content: '' } },
    { label: 'configs', path: `${sh.pwd().stdout}/src/configs/package.json`, options: { content: JSON.stringify({ name: '@configs' }) } },

    { label: 'routers', path: `${sh.pwd().stdout}/src/routers/index.js`, options: { content: '' } },
    { label: 'routers', path: `${sh.pwd().stdout}/src/routers/package.json`, options: { content: JSON.stringify({ name: '@routers' }) } },

    { label: 'db', path: `${sh.pwd().stdout}/src/db/index.js`, options: { content: '' } },
    { label: 'db', path: `${sh.pwd().stdout}/src/db/package.json`, options: { content: JSON.stringify({ name: '@db' }) } },
    { label: 'db', path: `${sh.pwd().stdout}/src/db/models/index.js`, options: { content: '' } },
    { label: 'db', path: `${sh.pwd().stdout}/src/db/migrations/index.js`, options: { content: '' } },
    { label: 'db', path: `${sh.pwd().stdout}/src/db/seeds/index.js`, options: { content: '' } },

    { label: 'core', path: `${sh.pwd().stdout}/src/core/index.js`, options: { content: '' } },
    { label: 'core', path: `${sh.pwd().stdout}/src/core/package.json`, options: { content: JSON.stringify({ name: '@core' }) } },

    { label: 'utils', path: `${sh.pwd().stdout}/src/utils/index.js`, options: { content: '' } },
    { label: 'utils', path: `${sh.pwd().stdout}/src/utils/package.json`, options: { content: JSON.stringify({ name: '@utils' }) } },

    { label: 'debugging', path: `${sh.pwd().stdout}/debugging/index.js`, options: { content: '' } },
    { label: 'debugging', path: `${sh.pwd().stdout}/debugging/package.json`, options: { content: JSON.stringify({ name: '@debugging' }) } },
  ];

  listFiles.forEach(element => {
    console.log(colorsTerminal.green('✔'), '[Source]', element.label);
    try {
      fs.writeFileSync(element.path, element.options.content);
    } catch (err) {
      console.warn(err)
    }
  });
}
async function main() {
  if (fs.existsSync(sh.pwd().stdout + "/" + name)) {
    await inquirer
      .prompt({
        type: 'confirm',
        name: 'DirectoryExist',
        message: 'Directory test already exists. Continue?',
        default: false
      }).then(answers => {
        if (!answers['DirectoryExist']) {
          sh.exit();
        } else {
          sh.exec(`rm -rf ${sh.pwd().stdout + "/" + name}`);
        }
      });
  }

  sh.exec(`mkdir ${name}`);
  sh.cd(name)
  
  await createFolder();
  await createFiles();

  sh.exec(`npm init`)

  console.log(colorsTerminal.green('Done!'));
}
main();